breeds [ agents cops ]
globals [ k                      ; Factor for determining arrest probability
          threshold              ; By how much must G > N to make someone rebel?
          grievance-color        ; what color (in the middle of an agent shape) represents anger with the government?
          cops-color             ; what color are the cops?  Defaults to black
          ]

agents-own [ risk-aversion       ; R, fixed for the agent's lifetime, ranging from 0-1 (inclusive)
             perceived-hardship  ; H, also ranging from 0-1 (inclusive)
             active?             ; if true, then the agent is actively rebelling
             jail-term           ; How many turns in jail remain? (If 0, the agent is not in jail.)
            ]

patches-own [ neighborhood                       ; surrounding patches within the vision
              active-agents-in-neighborhood      ; active agents are in the neighborhood
              ]

to setup
  ca

  ;; set globals
  set k 2.7
  set threshold 0.1
  set grievance-color green
  set-default-shape cops "person soldier"
  set cops-color "blue"

  ;; set background checkerboard
  ask patches [
    set pcolor orange + 3 + ( (pxcor + pycor) mod 2 )
  ]

  ;; cache patch neighborhoods
  ask patches [
    set neighborhood patches in-radius vision
  ]

  ;; create cops
  let number-of-cops round (initial-cop-density * .01 * count patches)
  create-custom-cops number-of-cops [
    move-to random-one-of patches
    set color black
  ]

  ;; create agents
  create-custom-agents round (initial-agent-density * .01 * count patches) [
    move-to random-one-of patches
    set heading 0
    set risk-aversion random 101 / 100
    set perceived-hardship random 101 / 100
    set active? false
    set jail-term 0
    display-state
  ]
end

to go
  ; Rule M: Move to a random site within your vision
  ask agents with [jail-term = 0] [ move ]
  ask cops [ cop-move ]

  ;; cache certain values on inhabited patches
  ask patches with [ any? turtles-here ] [
    set active-agents-in-neighborhood (agents-on (neighborhood)) with [ active? ]
  ]

  ; Rule A: Determine if each agent should be active or quiet
  ask agents with [jail-term = 0] [ determine-behavior ]

  ; Rule C: Cops arrest a random active agent within their radius
  ask cops [ enforce ]

  ; Jailed agents get their term reduced at the end of each clock tick
  ask agents [
    display-state
    if jail-term > 0 [ set jail-term (jail-term - 1) ]
  ]

  do-plot
end


;; AGENT BEHAVIOR

to display-state
      set color scale-color grievance-color grievance 1 0

      ;; Set an agent's state based on his behavior
      set shape "person quiet"

      if active?
      [ set shape "person active" ]

      if jail-term > 0
      [ set shape "person jailed" ]
end

to move ;; turtle procedure
  if ( movement? ) [
    ;; move to a patch in vision
    move-to random-one-of neighborhood
  ]
end

to move-to [dest]  ;; turtle procedure
  ;; go to random spot not too far from patch center,
  ;; so turtles don't always exactly overlap visually,
  ;; but you can still easily see which patches they are on
  setxy (pxcor-of dest) - 0.3 + random-float 0.6
        (pycor-of dest) - 0.3 + random-float 0.6
end

to determine-behavior
  set active? (grievance - net-risk > threshold)
end

to-report grievance
  report (perceived-hardship * (1 - government-legitimacy))
end

to-report estimated-arrest-probability
  report 1 - exp ( - k * (count cops-on neighborhood / (1 + count active-agents-in-neighborhood)))
end

to-report net-risk
  report risk-aversion * estimated-arrest-probability
end


;; COP BEHAVIOR

to enforce
  without-interruption [
    if (count active-agents-in-neighborhood > 0) [
      ;; arrest suspect
      let suspect random-one-of active-agents-in-neighborhood
      set active?-of suspect false
      set jail-term-of suspect random max-jail-term
    ]
  ]
end

to cop-move
    let dest random-one-of neighborhood
    setxy ( pxcor-of dest ) ( pycor-of dest )
    ;; go to random spot on this patch, so agents don't always overlap
    set xcor pxcor - 0.5 + random-float 1
    set ycor pycor - 0.5 + random-float 1
end

to do-plot
  set-current-plot "Agent types"
  set-current-plot-pen "Active"
  plot count agents with [active?]

  set-current-plot-pen "Jailed"
  plot count agents with [jail-term > 0]

  set-current-plot-pen "Quiet"
  plot count agents with [active? = false and jail-term = 0]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the projects:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS and INTEGRATED SIMULATION AND MODELING ENVIRONMENT.
; The project gratefully acknowledges the support of the
; National Science Foundation (REPP & ROLE programs) -- grant numbers
; REC #9814682 and REC-0126227.
;
; Copyright 2004 by Uri Wilensky.  Updated 2004.  All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (2004).  NetLogo Rebellion model.
; http://ccl.northwestern.edu/netlogo/models/Rebellion.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; In other publications, please use:
; Copyright 2004 Uri Wilensky.  All rights reserved.
; See http://ccl.northwestern.edu/netlogo/models/Rebellion
; for terms of use.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
325
10
909
615
20
20
14.0
1
10
1
1
1
0
1
1
1

CC-WINDOW
5
629
918
724
Command Center
0

BUTTON
10
248
76
281
NIL
setup
NIL
1
T
OBSERVER
T
NIL

BUTTON
89
248
152
281
NIL
go
T
1
T
OBSERVER
T
NIL

SLIDER
10
290
210
323
government-legitimacy
government-legitimacy
0
1
0.18
0.01
1
NIL

SLIDER
10
325
212
358
max-jail-term
max-jail-term
0
50
5
1
1
turns

MONITOR
216
418
305
467
Active (red)
count agents with [ active? ]
3
1

SLIDER
6
109
181
142
vision
vision
0
10
3
1
1
patches

MONITOR
122
418
215
467
Jailed (gray)
count agents with [ jail-term > 0 ]
3
1

SLIDER
7
31
211
64
initial-cop-density
initial-cop-density
0
100
11
1
1
%

MONITOR
13
418
117
467
Quiet (blue)
count agents with [ not active? and jail-term = 0 ]
3
1

SWITCH
10
363
149
396
movement?
movement?
0
1
-1000

MONITOR
96
154
179
203
# of cops
count cops
3
1

SLIDER
6
70
211
103
initial-agent-density
initial-agent-density
0
100
50
1
1
%

MONITOR
8
153
90
202
# of agents
count agents
3
1

PLOT
13
472
306
608
Agent types
Time
Agents
0.0
100.0
0.0
500.0
true
true
PENS
"Active" 1.0 0 -955883 true
"Jailed" 1.0 0 -7500403 true
"Quiet" 1.0 0 -13345367 true

TEXTBOX
7
10
97
28
Initial settings

@#$#@#$#@
WHAT IS IT?
-----------
This project models the rebellion of a subjugated population against a central authority.

The population wonders around randomly. If their level of grievance against the central authority is high enough, and their perception of the risks involved are low enough, they openly rebel. A separate population of cops, acting on behalf of the central authority, seeks to suppress the rebellion. The cops wander around randomly and arrest people who are actively rebelling.


HOW IT WORKS
------------
This model is an adaptation of Joshua Epstein's model of civil violence (2002). Each "agent," or member of the general population, has an individual level of grievance toward the central authority. GRIEVANCE is based on the agent's PERCEIVED-HARDSHIP, which is assigned randomly at startup, and 1 - GOVERNMENT-LEGITIMACY, which is global across agents and specified in the interface.

Each agent also calculates an individual risk of rebelling at the beginning of each turn.  This ESTIMATED-ARREST-PROBABILITY, is based on the number of cops and already rebelling agents within VISION patches, namely 1 - exp [ - k * (C/A)_v ] -- where (C/A)_v is the ratio of cops to active agents, and k is a constant set in "startup" to ensure a reasonable value when there is only one cop and one agent within a particular field of vision.

An agent's NET-RISK is the product of ESTIMATED-ARREST-PROBABILITY and RISK-AVERSION randomly set for each agent.

The model cycles through three different rules, known as A (agents), C (cops), and M (move):

- Agent rule A says that if an agent's GRIEVANCE exceeds the NET-RISK by a small threshold (set by default to 0.1 in "startup"), the agent decides to rebel.  This is noted by changing the agent's border color from blue to red.  (It is also possible for the agent to move from rebellion to quiet, in which case the border color changes from red to blue.)

- Cop rule C says that each cop should look for active agents within VISION patches.  If at least one exists, the cop randomly selects one active agent and sends it to jail for a number of turns between 0 and MAX-JAIL-TERM (settable in the interface).

- Movement rule M says that each cop and non-imprisoned agent moves to a random site within VISION patches.


HOW TO USE IT
-------------
Use the sliders to pick the initial settings for the model. INITIAL-COP-DENSITY and POPULATION-DENSITY respectively determine the density of cops and agents on the world. VISION determines the number of patches in each direction that agents and cops can see.
Click SETUP to initialize the population. Click GO to begin the simulation.

As the simulation runs, you can adjust GOVERNMENT-LEGITIMACY and MAX-JAIL-TERM. You can control the mobility of the agents through the MOVEMENT? toggle.

The color of the agents' faces shows their private level of grievance. The darker the green, the higher the GRIEVANCE.  This color does not change throughout the run of a model.


THINGS TO NOTICE
----------------
Watch how agents exhibit deceptive behavior by changing their public state when cops are present, despite a constant level of private grievance. To see this, slow down the model, type "watch random-one-of agents" into the command center, and switch to the 3D view and zoom in on the selected agent.   


THINGS TO TRY
-------------
Compare the effects of a gradual erosion in the perceived legitimacy of the central authority to a smaller, but immediate drop. For example, set GOVERNMENT-LEGITIMACY to 0.9 and reduce it to zero over 250 runs, plotting the legitimacy and the number of active agents. Now do another run, beginning with legitimacy at 0.9. This time, leave the legitimacy constant for 80 runs and then drop it to 0.7 at once. Observe the results. Try to explain them. Think of historical examples of this phenomena. What are the implications of this result for the strategy of revolutionary leaders?

Epstein quotes de Tocqueville: "It is not always when things are going from bad to worse that revolutions break out. On the contrary, it oftener happens that when a people that has put up with oppressive rule over a long period without protest suddenly finds the government relaxing its pressure, it takes up arms against it." Test de Tocqueville's observation in our model by comparing the effects of a gradual erosion in perceived legitimacy with a gradual erosion in the number of cops. Observe the results. Explain the difference.


EXTENDING THE MODEL
-------------------
Change GOVERNMENT-LEGITIMACY such that it increases proportionally with the number of nearby jailed agents.

Change the model such that each agent's GRIEVANCE is influenced by the value of other nearby agents.


NETLOGO FEATURES
----------------
In order to display both the private level of grievance and the public action of each of the agents, this project uses the key-color feature of the shapes editor.

In order to get random values that vary between 0 and 1 (inclusive), we use "random 101", and then divide the results by 100.


CREDITS AND REFERENCES
----------------------
This model, and the preceding explanation, is adapted from Joshua M. Epstein, "Modeling civil violence: An agent-based computational approach", Proceedings of the National Academy of Sciences, Vol. 99, Suppl. 3, May 14, 2002, and is available at http://www.pnas.org/cgi/content/abstract/99/suppl_3/7243 

To refer to this model in academic publications, please use: Wilensky, U. (2004). NetLogo Rebellion model. http://ccl.northwestern.edu/netlogo/models/Rebellion. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.

In other publications, please use: Copyright 2004 Uri Wilensky. All rights reserved. See http://ccl.northwestern.edu/netlogo/models/Rebellion for terms of use.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

agent active
false
0
Rectangle -2674135 true false 1 2 298 299
Circle -7500403 true true 26 26 247

agent jailed
false
0
Rectangle -6459832 true false 1 2 298 299
Circle -7500403 true true 26 26 247

agent quiet
false
0
Rectangle -13345367 true false 1 2 298 299
Circle -7500403 true true 26 26 247

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

person active
false
0
Polygon -13791810 true false 135 90 150 105 135 165 150 180 165 165 150 105 165 90
Polygon -2674135 true false 195 135 240 30 210 15 165 120
Circle -7500403 true true 110 5 80
Rectangle -7500403 true true 127 79 172 94
Polygon -2674135 true false 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Polygon -2674135 true false 105 135 60 30 90 15 135 120
Polygon -6459832 true false 195 15 270 60 270 75 195 30

person jailed
false
0
Circle -7500403 true true 110 5 80
Polygon -6459832 true false 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -6459832 true false 195 90 210 150 195 180 165 105
Polygon -6459832 true false 105 90 90 150 105 180 135 105

person quiet
false
0
Polygon -13791810 true false 135 90 150 105 135 165 150 180 165 165 150 105 165 90
Polygon -13345367 true false 195 90 240 195 210 210 165 105
Circle -7500403 true true 110 5 80
Rectangle -7500403 true true 127 79 172 94
Polygon -13345367 true false 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Polygon -1 true false 100 210 130 225 145 165 85 135 63 189
Polygon -13791810 true false 90 210 120 225 135 165 67 130 53 189
Polygon -1 true false 120 224 131 225 124 210
Line -16777216 false 139 168 126 225
Line -16777216 false 140 167 76 136
Polygon -13345367 true false 105 90 60 195 90 210 135 105

person soldier
false
5
Rectangle -7500403 true false 127 79 172 94
Polygon -10899396 true true 105 90 60 195 90 210 135 105
Polygon -10899396 true true 195 90 240 195 210 210 165 105
Circle -7500403 true false 110 5 80
Polygon -10899396 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Polygon -6459832 true false 120 90 105 90 180 195 180 165
Line -6459832 false 109 105 139 105
Line -6459832 false 122 125 151 117
Line -6459832 false 137 143 159 134
Line -6459832 false 158 179 181 158
Line -6459832 false 146 160 169 146
Rectangle -6459832 true false 120 193 180 201
Polygon -6459832 true false 122 4 107 16 102 39 105 53 148 34 192 27 189 17 172 2 145 0
Polygon -16777216 true false 183 90 240 15 247 22 193 90
Rectangle -6459832 true false 114 187 128 208
Rectangle -6459832 true false 177 187 191 208

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 3.0beta6
@#$#@#$#@
setup
repeat 5 [ go ]
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
